<script setup>
import { getCurrentInstance, ref, computed, reactive, watch } from 'vue';
import { useRouter } from "vue-router";
import { get, post, postBlob } from '@/utils/request.js'
import { message, Modal } from 'ant-design-vue';
import { logout } from '@/config.js';
import EditView from './edit.vue'
import { formatMessage } from '@/utils/localeUtils.js';
import useTableSearch from '@/utils/tableUtils.jsx';
import OrderOutWavePreviewView from '@/components/orderOut/orderOutWavePreview/index.vue';
const context = getCurrentInstance()?.appContext.config.globalProperties;

const router = useRouter();

const TABLE_KEY = 'table-wave';
let currentPage = 1;
let pageSize = 10;
const searchInput = ref("")
const searchParam = ref({
  waveNo: null,
  matnr: null,
  batch: null,
})
const editChild = ref(null)
const orderOutWavePreviewChild = ref(null);

const state = reactive({
  selectedRowKeys: [],
  loading: false,
  columns: [],
});

let tableData = ref([]);
getPage();

const {
  getColumnSearchProps,
  handleResizeColumn,
} = useTableSearch();

let tableDataColSpan = []
const customColSpanProps = (index) => {
  let count = tableDataColSpan[index];
  if (count == null) {
    count = 0;
  }
  return {
    rowSpan: count,
  };
}

state.columns = [
  {
    title: formatMessage('db.man_wave.wave_no', '波次号'),
    dataIndex: 'waveNo',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('waveNo'),
    customCell: (_, index) => {
      return customColSpanProps(index)
    },
  },
  {
    title: formatMessage('db.man_wave.waveType$', '类型'),
    dataIndex: 'waveType$',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('waveType$'),
    customCell: (_, index) => {
      return customColSpanProps(index)
    },
  },
  {
    title: formatMessage('db.man_wave.waveStatus$', '状态'),
    dataIndex: 'waveStatus$',
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('waveStatus$'),
    customCell: (_, index) => {
      return customColSpanProps(index)
    },
  },
  {
    title: formatMessage('db.man_wave.matnr', '商品编号'),
    dataIndex: ['mat$', 'matnr'],
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('matnr'),
  },
  {
    title: formatMessage('db.man_wave.maktx', '商品名称'),
    dataIndex: ['mat$', 'maktx'],
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('maktx'),
  },
  {
    title: formatMessage('db.man_wave.batch', '批号'),
    dataIndex: ['batch'],
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('batch'),
  },
  {
    title: formatMessage('db.man_wave.anfme', '数量'),
    dataIndex: ['anfme'],
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('anfme'),
  },
  {
    title: formatMessage('db.man_wave.work_qty', '工作数量'),
    dataIndex: ['workQty'],
    width: 140,
    ellipsis: true,
    ...getColumnSearchProps('workQty'),
  },

];

const hasSelected = computed(() => state.selectedRowKeys.length > 0);
const start = () => {
  state.loading = true;
  // ajax request after empty completing
  setTimeout(() => {
    state.loading = false;
    state.selectedRowKeys = [];
  }, 1000);
};
const onSelectChange = selectedRowKeys => {
  // console.log('selectedRowKeys changed: ', selectedRowKeys);
  state.selectedRowKeys = selectedRowKeys;
};

function getPage() {
  state.loading = true;

  post('/api/wave/page', {
    current: currentPage,
    pageSize: pageSize,
    condition: searchInput.value,
    _param: searchParam.value,
  }).then((resp) => {
    let result = resp.data;
    if (result.code == 200) {
      let data = result.data.records;

      let tmp = []
      let idx = 0;
      let colSpan = []
      data.forEach((item) => {
        let count = 0;

        item.waveDetlList.forEach((detl) => {
          detl.waveStatus = item.waveStatus;
          detl.waveStatus$ = item.waveStatus$;
          detl.waveType = item.waveType;
          detl.waveType$ = item.waveType$;

          count++;
          tmp.push(detl)
        })

        colSpan[idx] = count;
        idx += count;
      })

      tableDataColSpan = colSpan;
      tableData.value = tmp;

      state.loading = false;

    } else if (result.code === 401) {
      message.error(result.msg);
      logout()
    } else {
      message.error(result.msg);
    }
  })
}

const fieldList = [];
getColumns();
//加载扩展字段
async function getColumns() {
  let fieldResp = await post('/api/matField/list', {
    unique: 1
  })
  let fieldResult = fieldResp.data;
  let tmp = state.columns;
  if (fieldResult.code == 200) {
    let data = fieldResult.data;

    data.forEach((item) => {
      tmp.push({
        title: formatMessage(item.language, item.describe),
        name: item.name,
        dataIndex: item.name,
        key: item.name,
        width: 140,
        editable: true,
      })

      fieldList.push(item.name);
    })

    tmp.push({
      title: formatMessage('common.operation', '操作'),
      name: 'oper',
      dataIndex: 'oper',
      key: 'oper',
      width: 200,
      fixed: 'right',
      customCell: (_, index) => {
        return customColSpanProps(index)
      },
    })

    state.columns = tmp;

  } else if (result.code === 401) {
    message.error(result.msg);
    logout()
  } else {
    message.error(result.msg);
  }
}

// const handleEdit = (item) => {
//   editChild.value.open = true;
//   editChild.value.formData = item == null ? editChild.value.initFormData : JSON.parse(JSON.stringify(item));
//   editChild.value.isSave = item == null;
// }

const handleGenerateOut = (waveId) => {
  orderOutWavePreviewChild.value.open = true;
  orderOutWavePreviewChild.value.waveId = waveId;
}

const handleDel = (rows) => {
  Modal.confirm({
    title: formatMessage('page.cancel', '取消'),
    content: formatMessage('page.cancel.confirm', '确定取消该项吗？'),
    maskClosable: true,
    onOk: async () => {
      const hide = message.loading(formatMessage('common.loading', '请求中'));
      try {
        let waveIds = []
        rows.forEach((item) => {
          waveIds.push(item.waveId)
        })

        post('/api/out/wave/cancel', waveIds).then(resp => {
          let result = resp.data;
          if (result.code === 200) {
            message.success(result.msg);
          } else {
            message.error(result.msg);
          }
          getPage()
          hide()
        })
      } catch (error) {
        message.error(formatMessage('common.fail', '请求失败'));
      }
    },
  });
}

const handleExport = async (intl) => {
  postBlob('/api/wave/export', {}).then(result => {
    const blob = new Blob([result.data], { type: 'application/vnd.ms-excel' });
    window.location.href = window.URL.createObjectURL(blob);
    return true;
  })
};

const onSearch = () => {
  // console.log('search');
  getPage()
}

const onPageChange = (page, size) => {
  currentPage = page;
  pageSize = size;
  getPage();
}

function handleTableReload(value) {
  getPage()
}

const handleChildReload = () => {
  handleTableReload()
}

</script>

<script>
export default {
  name: '波次管理'
}
</script>

<template>
  <div>
    <EditView ref="editChild" @tableReload="handleTableReload" />
    <div class="table-header">
      <div>
        <a-input v-model:value="searchParam.waveNo" :placeholder="formatMessage('page.wave.waveNo.input', '请输入波次号')"
          style="width: 140px;margin-right: 10px;" />
        <a-input v-model:value="searchParam.matnr" :placeholder="formatMessage('page.wave.matnr.input', '请输入商品编号')"
          style="width: 140px;margin-right: 10px;" />
        <a-input v-model:value="searchParam.batch" :placeholder="formatMessage('page.wave.batch.input', '请输入批号')"
          style="width: 140px;margin-right: 10px;" />

        <a-input-search v-model:value="searchInput" :placeholder="formatMessage('page.input', '请输入')"
          style="width: 200px;" @search="onSearch" />
      </div>
      <!-- <div class="table-header-right">
        <a-button @click="handleEdit(null)" type="primary">{{ formatMessage('page.add', '添加') }}</a-button>
        <a-button @click="handleExport">{{ formatMessage('page.export', '导出') }}</a-button>
      </div> -->
    </div>
    <a-table :data-source="tableData" :defaultExpandAllRows="false" :key="TABLE_KEY" rowKey="index"
      :scroll="{ y: 768, scrollToFirstRowOnChange: true }" :columns="state.columns" @resizeColumn="handleResizeColumn"
      :loading="state.loading">
      <template #bodyCell="{ column, text, record }">
        <template v-if="column.dataIndex === 'oper'">
          <div style="display: flex;justify-content: space-evenly;">
            <a-button type="link" primary :disabled="record.waveStatus != 0"
              @click="handleGenerateOut(record.waveId)">{{
                formatMessage('page.generateOut',
                  '生成出库任务')
              }}</a-button>
            <a-button type="link" danger @click="handleDel([record])">{{ formatMessage('page.cancel', '取消')
              }}</a-button>
          </div>
        </template>
      </template>
    </a-table>

    <OrderOutWavePreviewView ref="orderOutWavePreviewChild" @reload="handleChildReload" />
  </div>
</template>

<style></style>
